@@ -35,6 +35,7 @@ To enable the magics below, execute ``%load_ext rmagic``.
 {RGET_DOC}
 
 """
+from __future__ import print_function
 
 #-----------------------------------------------------------------------------
 #  Copyright (C) 2012 The IPython Development Team
@@ -72,7 +73,9 @@ from IPython.core.magic_arguments import (
     argument, magic_arguments, parse_argstring
 )
 from IPython.external.simplegeneric import generic
-from IPython.utils.py3compat import str_to_unicode, unicode_to_str, PY3
+from IPython.utils.py3compat import (str_to_unicode, unicode_to_str, PY3,
+                                     unicode_type)
+from IPython.utils.text import dedent
 
 class RInterpreterError(ri.RRuntimeError):
     """An error when running R code in a %%R magic cell."""
@@ -193,20 +196,28 @@ class RMagics(Magics):
 
     def eval(self, line):
         '''
-        Parse and evaluate a line with rpy2.
-        Returns the output to R's stdout() connection
-        and the value of eval(parse(line)).
+        Parse and evaluate a line of R code with rpy2.
+        Returns the output to R's stdout() connection, 
+        the value generated by evaluating the code, and a
+        boolean indicating whether the return value would be
+        visible if the line of code were evaluated in an R REPL.
+        
+        R Code evaluation and visibility determination are
+        done via an R call of the form withVisible({<code>})
+        
         '''
         old_writeconsole = ri.get_writeconsole()
         ri.set_writeconsole(self.write_console)
         try:
-            value = ri.baseenv['eval'](ri.parse(line))
+            res = ro.r("withVisible({%s})" % line)
+            value = res[0] #value (R object)
+            visible = ro.conversion.ri2py(res[1])[0] #visible (boolean)
         except (ri.RRuntimeError, ValueError) as exception:
             warning_or_other_msg = self.flush() # otherwise next return seems to have copy of error
             raise RInterpreterError(line, str_to_unicode(str(exception)), warning_or_other_msg)
         text_output = self.flush()
         ri.set_writeconsole(old_writeconsole)
-        return text_output, value
+        return text_output, value, visible
 
     def write_console(self, output):
         '''
@@ -382,7 +393,7 @@ class RMagics(Magics):
         help='Convert these objects to data.frames and return as structured arrays.'
         )
     @argument(
-        '-u', '--units', type=unicode, choices=["px", "in", "cm", "mm"],
+        '-u', '--units', type=unicode_type, choices=["px", "in", "cm", "mm"],
         help='Units of png plotting device sent as an argument to *png* in R. One of ["px", "in", "cm", "mm"].'
         )
     @argument(
@@ -422,12 +433,15 @@ class RMagics(Magics):
             In [9]: %R X=c(1,4,5,7); sd(X); mean(X)
             Out[9]: array([ 4.25])
 
-        As a cell, this will run a block of R code, without bringing anything back by default::
+        In cell mode, this will run a block of R code. The resulting value
+        is printed if it would printed be when evaluating the same code 
+        within a standard R REPL.
+        
+        Nothing is returned to python by default in cell mode::
 
             In [10]: %%R
                ....: Y = c(2,4,3,9)
-               ....: print(summary(lm(Y~X)))
-               ....:
+               ....: summary(lm(Y~X))
 
             Call:
             lm(formula = Y ~ X)
@@ -445,9 +459,9 @@ class RMagics(Magics):
             Multiple R-squared: 0.6993,Adjusted R-squared: 0.549
             F-statistic: 4.651 on 1 and 2 DF,  p-value: 0.1638
 
-        In the notebook, plots are published as the output of the cell.
+        In the notebook, plots are published as the output of the cell::
 
-        %R plot(X, Y)
+            %R plot(X, Y)
 
         will create a scatter plot of X bs Y.
 
@@ -589,14 +603,20 @@ class RMagics(Magics):
         try:
             if line_mode:
                 for line in code.split(';'):
-                    text_result, result = self.eval(line)
+                    text_result, result, visible = self.eval(line)
                     text_output += text_result
                 if text_result:
                     # the last line printed something to the console so we won't return it
                     return_output = False
             else:
-                text_result, result = self.eval(code)
+                text_result, result, visible = self.eval(code)
                 text_output += text_result
+                if visible:
+                    old_writeconsole = ri.get_writeconsole()
+                    ri.set_writeconsole(self.write_console) 
+                    ro.r.show(result)
+                    text_output += self.flush()
+                    ri.set_writeconsole(old_writeconsole)
         
         except RInterpreterError as e:
             print(e.stdout)
@@ -660,10 +680,10 @@ class RMagics(Magics):
                 return self.Rconverter(result, dataframe=False)
 
 __doc__ = __doc__.format(
-                R_DOC = ' '*8 + RMagics.R.__doc__,
-                RPUSH_DOC = ' '*8 + RMagics.Rpush.__doc__,
-                RPULL_DOC = ' '*8 + RMagics.Rpull.__doc__,
-                RGET_DOC = ' '*8 + RMagics.Rget.__doc__
+                R_DOC = dedent(RMagics.R.__doc__),
+                RPUSH_DOC = dedent(RMagics.Rpush.__doc__),
+                RPULL_DOC = dedent(RMagics.Rpull.__doc__),
+                RGET_DOC = dedent(RMagics.Rget.__doc__)
 )
 
 
