@@ -1,5 +1,5 @@
 //----------------------------------------------------------------------------
-//  Copyright (C) 2008-2011  The IPython Development Team
+//  Copyright (C) 2011  The IPython Development Team
 //
 //  Distributed under the terms of the BSD License.  The full license is in
 //  the file COPYING, distributed as part of this software.
@@ -7,7 +7,11 @@
 
 var IPython = IPython || {};
 
+IPython.version = "2.0.0";
+
 IPython.namespace = function (ns_string) {
+    "use strict";
+
     var parts = ns_string.split('.'),
         parent = IPython,
         i;
